@@ -1,6 +1,6 @@
 /* Definitions of Aris' processing engine.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,23 +23,52 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+#include "typedef.h"
 
-#ifndef WIN32
-#include <libintl.h>
-#define _(String) gettext (String)
-#define gettext_noop(String) String
-#define N_(String) gettext_noop (String)
-#else
-#define _(String) String
-#define N_(String) String
-#endif
-
-#define REPORT() printf ("Line %i reporting!\n", __LINE__);
-#define CHECK_ALLOC(o,r) if (!o) {perror (NULL); return r; }
 
 #define ISLEGIT(c) (islower (c) || isdigit (c) || c == '_')
+
 #define ISSEP(c) (c == ')' || c == ',' || c == '+' || c == '*' || c == '(')
-#define ISGOOD(s) (!strncmp (s, UNV, CL) || !strncmp (s, EXL, CL) || *(s) == '(' || isupper (*(s)) || !strncmp (s, NOT, NL) || !strncmp (s, CTR, CL) || !strncmp (s, TAU, CL))
+
+#define IS_TYPE_CONN(s,t) (!strncmp (s, t.and, t.cl)		\
+			   || !strncmp (s, t.or, t.cl)		\
+			   || !strncmp (s, t.not, t.nl)		\
+			   || !strncmp (s, t.con, t.cl)		\
+			   || !strncmp (s, t.bic, t.cl)		\
+			   || !strncmp (s, t.unv, t.cl)		\
+			   || !strncmp (s, t.exl, t.cl)		\
+			   || !strncmp (s, t.tau, t.cl)		\
+			   || !strncmp (s, t.ctr, t.cl)		\
+			   || !strncmp (s, t.elm, t.cl)		\
+			   || !strncmp (s, t.nil, t.cl))
+
+#define ISCONN(s) IS_TYPE_CONN (s, main_conns)
+
+#define ISGOOD(s) (!strncmp (s, UNV, CL)     \
+		   || !strncmp (s, EXL, CL)  \
+		   || *(s) == '('	     \
+		   || isupper (*(s))	     \
+		   || !strncmp (s, NOT, NL)  \
+		   || !strncmp (s, CTR, CL)  \
+		   || !strncmp (s, TAU, CL))
+
+#define ISSBOOL(s) (!strncmp (s, S_TAU, S_CL) || !strncmp (s, S_CTR, S_CL))
+
+
+enum CONN_ORDER {
+  AND_CONN=0,
+  OR_CONN,
+  NOT_CONN,
+  CON_CONN,
+  BIC_CONN,
+  UNV_CONN,
+  EXL_CONN,
+  TAU_CONN,
+  CTR_CONN,
+  ELM_CONN,
+  NIL_CONN,
+  NUM_CONNS
+};
 
 // The struct for connectives handling.
 
@@ -67,21 +96,15 @@ static struct connectives_list cli_conns = {
 
 // GUI connectives.
 
+// Windows still seems to refuse to display these properly.
+// And I'm still out of ideas as to why.
+// wxWidgets got it to work, although I'm not entirely certain how.
+// Something about the unicode support.
+// It also works just fine under Wine, which is why it's hard to test it.
+// Gtk only displays utf-8 strings.  Maybe that has something to do
+//  with it.
+
 static struct connectives_list gui_conns = {
-#ifndef WIN32
-  "\342\210\247",
-  "\342\210\250",
-  "\302\254",
-  "\342\206\222",
-  "\342\206\224",
-  "\342\210\200",
-  "\342\210\203",
-  "\342\212\244",
-  "\342\212\245",
-  "\342\210\210",
-  "\342\215\211",
-  3, 2
-#else
   "\u2227",
   "\u2228",
   "\u00AC",
@@ -93,8 +116,26 @@ static struct connectives_list gui_conns = {
   "\u22A5",
   "\u2208",
   "\u2349",
-  1, 1
-#endif
+  3, 2
+};
+
+
+static const char * conn_list_back[] = {
+  "\u2227",
+  "\u2228",
+  "\u00AC",
+  "\u2192",
+  "\u2194",
+  "\u2200",
+  "\u2203",
+  "\u22A4",
+  "\u22A5",
+  "\u2208",
+  "\u2349"
+};
+
+static const char * conn_list[] = {
+  "&", "|", "~", "$", "%", "@", "#", "!", "^", ":", ">"
 };
 
 // Sexpr connectives.
@@ -181,12 +222,6 @@ struct sen_id {
   int id;
 };
 
-typedef struct sen_id sen_id;
-typedef struct vector vec_t;
-typedef struct proof proof_t;
-typedef struct sen_data sen_data;
-typedef struct list list_t;
-
 /* Parse functions. */
 
 int parse_parens (const unsigned char * in_str,
@@ -197,6 +232,11 @@ int reverse_parse_parens (const unsigned char * in_str,
 			  const int init_pos,
 			  unsigned char ** out_str);
 
+int parse_tags (const unsigned char * in_str,
+		const int init_pos,
+		unsigned char ** out_str,
+		const char * o_tag, const char * c_tag);
+
 /* Check functions. */
 
 int check_parens (const unsigned char * chk_str);
