@@ -80,21 +80,10 @@ proc_bi (unsigned char * prem, unsigned char * conc)
   // Standard long-short sentence setup.
 
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -210,8 +199,8 @@ proc_bi (unsigned char * prem, unsigned char * conc)
       return _("There must be a tautology or a contradiction in the generalities.");
     }
 
-  if ((!strcmp (conn, S_AND) && strcmp (good_side, S_TAU))
-      && (!strcmp (conn, S_OR) && strcmp (good_side, S_CTR)))
+  if ((strcmp (conn, S_AND) || strcmp (good_side, S_TAU))
+      && (strcmp (conn, S_OR) || strcmp (good_side, S_CTR)))
     {
       free (lsen);
       free (rsen);
@@ -222,17 +211,12 @@ with a disjunction.");
   free (good_side);
 
   unsigned char * oth_sen;
-  int oth_pos;
+  int alloc_size;
 
-  oth_sen = (unsigned char *) calloc (l_len - 2 * S_CL - 3, sizeof (char));
-  CHECK_ALLOC (oth_sen, NULL);
-  strncpy (oth_sen, ln_sen, li);
-  oth_pos = li;
-
-  oth_pos += sprintf (oth_sen + oth_pos, "%s", bad_side);
-  free (bad_side);
-
-  strcpy (oth_sen + oth_pos, ln_sen + tmp_pos + 1);
+  alloc_size = l_len - 2 * S_CL - 4;
+  oth_sen = construct_other (ln_sen, li, tmp_pos + 1, alloc_size, "%s", bad_side);
+  if (!oth_sen)
+    return NULL;
 
   char * ret_str;
 
@@ -255,21 +239,10 @@ char *
 proc_bd (unsigned char * prem, unsigned char * conc)
 {
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -313,6 +286,7 @@ proc_bd (unsigned char * prem, unsigned char * conc)
       return _("There must be a conjunction or a disjunction at the difference.");
     }
 
+  unsigned char sym[S_CL + 1];
   int j;
 
   for (j = 0; j < gens->num_stuff; j++)
@@ -321,10 +295,18 @@ proc_bd (unsigned char * prem, unsigned char * conc)
       cur_gen = vec_nth (gens, j);
 
       if (!strcmp (*cur_gen, S_CTR) && !strcmp (conn, S_AND))
-	    break;
+	{
+	  strncpy (sym, S_CTR, S_CL);
+	  sym[S_CL] = '\0';
+	  break;
+	}
 
       if (!strcmp (*cur_gen, S_TAU) && !strcmp (conn, S_OR))
-	    break;
+	{
+	  strncpy (sym, S_TAU, S_CL);
+	  sym[S_CL] = '\0';
+	  break;
+	}
     }
 
   destroy_str_vec (gens);
@@ -333,19 +315,12 @@ proc_bd (unsigned char * prem, unsigned char * conc)
     return _("There must be a tautology or a contradiction within the difference.");
 
   unsigned char * oth_sen;
-  int oth_pos;
+  int alloc_size;
+  alloc_size = l_len - t_len + S_CL;
 
-  oth_sen = (unsigned char *) calloc (l_len - t_len + S_CL + 1, sizeof (char));
-  CHECK_ALLOC (oth_sen, NULL);
-  strncpy (oth_sen, ln_sen, i);
-  oth_pos = i;
-
-  if (!strcmp (conn, S_AND))
-    oth_pos += sprintf (oth_sen + oth_pos, "%s", S_CTR);
-  else
-    oth_pos += sprintf (oth_sen + oth_pos, "%s", S_TAU);
-
-  strcpy (oth_sen + oth_pos, ln_sen + tmp_pos + 1);
+  oth_sen = construct_other (ln_sen, i, tmp_pos + 1, alloc_size, "%s", sym);
+  if (!oth_sen)
+    return NULL;
 
   char * ret_str;
 
@@ -368,21 +343,10 @@ char *
 proc_bn (unsigned char * prem, unsigned char * conc)
 {
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -426,24 +390,24 @@ proc_bn (unsigned char * prem, unsigned char * conc)
       return _("The second part must be the negation of the first.");
     }
 
-  unsigned char * oth_sen;
-  int oth_pos;
+  unsigned char sym[S_CL + 1];
+  if (!strcmp (conn, S_AND))
+    strncpy (sym, S_CTR, S_CL);
+  else
+    strncpy (sym, S_TAU, S_CL);
+  sym[S_CL] = '\0';
 
-  oth_sen = (unsigned char *) calloc (l_len - 2 * strlen (lsen) - S_NL - 6,
-				      sizeof (char));
-  CHECK_ALLOC (oth_sen, NULL);
-  free (lsen);
   free (rsen);
 
-  strncpy (oth_sen, ln_sen, i);
-  oth_pos = i;
+  unsigned char * oth_sen;
+  int alloc_size;
 
-  if (!strcmp (conn, S_AND))
-    oth_pos += sprintf (oth_sen + oth_pos, "%s", S_CTR);
-  else
-    oth_pos += sprintf (oth_sen + oth_pos, "%s", S_TAU);
+  alloc_size = l_len - 2 * strlen (lsen) - S_NL - 7;
+  free (lsen);
 
-  strcpy (oth_sen + oth_pos, ln_sen + tmp_pos + 1);
+  oth_sen = construct_other (ln_sen, i, tmp_pos + 1, alloc_size, "%s", sym);
+  if (!oth_sen)
+    return NULL;
 
   char * ret_str;
 
@@ -466,21 +430,10 @@ char *
 proc_sn (unsigned char * prem, unsigned char * conc)
 {
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -488,7 +441,7 @@ proc_sn (unsigned char * prem, unsigned char * conc)
     return NO_DIFFERENCE;
 
   if (ln_sen[i] != '(' || strncmp (ln_sen + i + 1, S_NOT, S_NL))
-    return _("There must be a negation at the difference.");
+    return _("There must be a negation in one sentence.");
 
   int tmp_pos;
   unsigned char * tmp_str, * elm_str;
@@ -505,24 +458,24 @@ proc_sn (unsigned char * prem, unsigned char * conc)
   if (strcmp (elm_str, S_CTR) && strcmp (elm_str, S_TAU))
     {
       free (elm_str);
-      return _("There must be a negated symbol at the difference.");
+      return _("There must be a negated symbol in one sentence.");
     }
 
-  unsigned char * oth_sen;
-  int oth_pos;
-
-  oth_sen = (unsigned char *) calloc (l_len - S_NL - 2, sizeof (char));
-  CHECK_ALLOC (oth_sen, NULL);
-  strncpy (oth_sen, ln_sen, i);
-  oth_pos = i;
-
+  unsigned char sym[S_CL + 1];
   if (!strcmp (elm_str, S_CTR))
-    oth_pos += sprintf (oth_sen + oth_pos, "%s", S_TAU);
+    strncpy (sym, S_TAU, S_CL);
   else
-    oth_pos += sprintf (oth_sen + oth_pos, "%s", S_CTR);
-
+    strncpy (sym, S_CTR, S_CL);
+  sym[S_CL] = '\0';
   free (elm_str);
-  strcpy (oth_sen + oth_pos, ln_sen + tmp_pos + 1);
+
+  unsigned char * oth_sen;
+  int alloc_size;
+
+  alloc_size = l_len - S_NL - 3;
+  oth_sen = construct_other (ln_sen, i, tmp_pos + 1, alloc_size, "%s", sym);
+  if (!oth_sen)
+    return NULL;
 
   char * ret_str;
 
